import { State } from '../state';
import { DataResult } from '../data-result.interface';
import { CompositeFilterDescriptor } from '../filtering/filter-descriptor.interface';
export type IdType = string | number;
export interface DataSource<T, TId extends IdType = never> {
    /**
     * Gets the table rows, applying the specified operation descriptors.
     * @param {State} state - The operation descriptors that will be applied to the data.
     */
    getData(params: State): Promise<DataResult<T>>;
    addData?(row: T, index?: number): Promise<T>;
    updateData?(id: TId, changes: Partial<T>): Promise<void>;
    removeData?(id: TId): Promise<T | undefined>;
    idSelector?(row: T): TId;
    /**
     * For persistent data stores (where objects persist across multiple getData calls),
     * gets all the data that is excluded by the filter. Undefined for non-persistent data stores.
     */
    getFilteredPersistentItems?(filter: CompositeFilterDescriptor): T[];
}
//# sourceMappingURL=data-source.d.ts.map