import { GroupResult } from '../grouping/group-descriptor.interface';
import { AggregateResult } from '../transducers';
/**
 * Represents the grouped result returned by the UI for ASP.NET MVC `ToDataSourceResult` method.
 */
export type ServerGroupResult = {
    /**
     * Contains either the subgroups or the data items.
     */
    Items: Object[];
    /**
     * The aggregated values for the group. An [`AggregateResult`]({% slug api_kendo-data-query_aggregateresult %}) instance.
     */
    Aggregates: any;
    /**
     * The field by which the data items are grouped.
     */
    Member: string;
    /**
     * The group key.
     */
    Key: any;
    /**
     * Determines if `Items` contains subgroups.
     */
    HasSubgroups: boolean;
};
/**
 * Converts the grouped result, which is returned into the `Data` field of the UI for ASP.NET MVC `ToDataSourceResult` method, to a comparable format.
 * @param data - The value of the `Data` field of the response.
 * @returns {GroupResult[]} - The converted result.
 */
export declare const translateDataSourceResultGroups: (data: any[]) => GroupResult[];
/**
 * Converts the `AggregateResults` field content, which is returned by the UI for ASP.NET MVC `ToDataSourceResult` method, to a comparable format.
 * @param data - The value of the `AggregateResults` field of the response.
 * @returns {AggregateResult} - The converted result.
 */
export declare const translateAggregateResults: (data: any[]) => AggregateResult;
//# sourceMappingURL=deserialization.d.ts.map