import { Predicate, Reducer, Transformer, TransformerResult, Preprocessors } from './common.interfaces';
/**
 * @hidden
 */
export declare const groupCombinator: <T>(field: any, preprocessors?: Preprocessors<T>) => (agg: any, value: any) => any;
/**
 * The result of the [aggregate operation]({% slug api_kendo-data-query_aggregateby %}) where the keys are the field name used for the calculation and the values are an object with the aggregate function name and a resulting value.
 *
 * @example
 * ```js-no-run
 * const result = <AggregateResult>{
 *      "unitPrice": { "sum": 53, "count": 2 },
 *      "unitsInStock": { "sum": 66, "count": 3 }
 * };
 *
 * console.log(result["unitPrice"].sum); //53
 *
 * ```
 */
export interface AggregateResult {
    [fieldName: string]: {
        count?: number;
        sum?: number;
        average?: number;
        min?: number;
        max?: number;
        count_distinct?: number;
    };
}
/**
 * @hidden
 */
export declare const expandAggregates: (result?: {}) => AggregateResult;
/**
 * @hidden
 */
export declare const aggregatesCombinator: <T>(descriptors: any, preprocessors?: Preprocessors<T>) => (state: any, value: any) => any;
/**
 * @hidden
 * Adds the value to the `arr` and produces a new array.
 *
 * > The original array will be modified.
 */
export declare const concat: <T>(arr: T[], value: T) => T[];
/**
 * @hidden
 * Returns a reducer that will apply the specified transformation to the value.
 */
export declare const map: (transform: any) => Reducer;
/**
 * @hidden
 * Returns a reducer that will filter out items which do not match the `Predicate`.
 */
export declare const filter: (predicate: any) => Reducer;
/**
 * @hidden
 */
export declare const isTransformerResult: <T>(source: TransformerResult<T>) => source is TransformerResult<T>;
/**
 * @hidden
 * Returns a reducer that will take the specified number of items.
 */
export declare const take: (count: number) => Reducer;
/**
 * @hidden
 * Returns a reducer that will take the specified number of items.
 */
export declare const takeWhile: (predicate: Predicate) => Reducer;
/**
 * @hidden
 * Returns a reducer that will skip the specified number of items.
 */
export declare const skip: (count: number) => Reducer;
/**
 * @hidden
 * Transforms the data by applying the supplied transformer.
 */
export declare const exec: <T, U>(transform: Transformer, initialValue: U, data: T[]) => U;
//# sourceMappingURL=transducers.d.ts.map